/*
    set.c

    Implementac. de conjuntos de cualquier longitud, en C

    Un conjunto es un vector de int's, tantos como sean necesarios para
    cubrir la diferencia entre el ĺim.inf. y el sup.

    El conjunto se almacena en memoria, en un solo vector. De este vector,
    el primer elemento es el lim. sup. y el segundo el lim. inf.

*/

#include "set.h"

#include <stdlib.h>
#include <memory.h>
#include <limits.h>

static SetStatus setStatus;
static const unsigned int NumFixedPositions = 2;

Set createSet(int l, int h)
{
    Set toret = NULL;
    setStatus = SetOk;

    if ( l >= h ) {
        setStatus = SetInvalidRange;
    } else {
        // avoid memory alignment problems
        int size = ( ( h - l ) / ( sizeof( unsigned int ) * CHAR_BIT )  )
                   + NumFixedPositions + 2
        ;

        toret = (Set) calloc( size, sizeof( unsigned int ) );

        // initialise struct
        if ( toret != NULL ) {
            *( (int *) toret ) = h;
            *( ( (int *) toret ) + 1 ) = l;
        } else {
            setStatus = SetNoMemory;
        }
    }

    return toret;
}

Set createFilledSet(int l, int h)
{
    Set toret = createSet( l, h );

    if ( getSetStatus() == SetOk ) {
        insertSetRange( toret, l, h );
    }

    return toret;
}

static inline
unsigned int * getBlock(Set s, unsigned int pos)
{
    pos /= sizeof( unsigned int ) * CHAR_BIT;
    return s + NumFixedPositions + pos;
}

static inline
unsigned int getPositionInBlock(Set s, unsigned int pos)
{
    return ( pos %= sizeof( unsigned int ) * CHAR_BIT );
}

bool isWithinBoundsOfSet(Set s, int pos)
{
    bool toret = true;

    if ( pos < getLowestElement( s )
      || pos > getHighestElement( s ) )
    {
        toret = false;
    }

    return toret;
}

bool querySetElement(Set s, int pos)
{
    bool toret = false;
    setStatus = SetOk;

    if ( isWithinBoundsOfSet( s, pos ) ) {
        unsigned int * blk = getBlock( s, pos );
        unsigned int bit = getPositionInBlock( s, pos );
        unsigned int mask = 1 << bit;

        toret = ( ( *blk ) & mask );
    } else setStatus = SetInvalidPosition;

    return toret;
}

void insertSetElement(Set s, int pos)
{
    setStatus = SetOk;

    if ( isWithinBoundsOfSet( s, pos ) ) {
        unsigned int * blk = getBlock( s, pos );
        unsigned int bit = getPositionInBlock( s, pos );
        unsigned int mask = 1 << bit;

        *blk |= mask;
    } else setStatus = SetInvalidPosition;
}

void deleteSetElement(Set s, int pos)
{
    setStatus = SetOk;

    if ( isWithinBoundsOfSet( s, pos ) ) {
        unsigned int * blk = getBlock( s, pos );
        unsigned int bit = getPositionInBlock( s, pos );
        unsigned int mask = 1 << bit;

        mask = ~mask;
        *blk &= mask;
    } else setStatus = SetInvalidPosition;
}

inline
SetStatus getSetStatus()
{
    return setStatus;
}

void deleteSetRange(Set s, int l, int h)
{
    int i;

    for(i = l; i <= h; ++i) {
        deleteSetElement( s, i );
    }
}

void insertSetRange(Set s, int l, int h)
{
    int i;

    for(i = l; i <= h; ++i) {
        insertSetElement( s, i );
    }
}

void destroySet(Set s)
{
    setStatus = SetOk;
    free( s );
}
