/* set.h
    Cabecera de mod. de conjuntos
*/

#ifndef SET_H_INCLUDED
#define SET_H_INCLUDED

/** Conjuntos */
typedef unsigned int * Set;

/** Errores posibles del conjunto
    <ol>
    <li>SetOk Significa que no se produjo ningún error.
    <li>SetInvalidRange El rango indicado no era válido
    <li>SetInvalidPosition La posición indicada no era válida
    <li>SetNoMemory No había suficiente memoria para crear el conjunto
    <p>Después de cada llamada a una función en este módulo, el valor de error
    del conjunto se actualiza, y se puede obtener mediante una llamada a getSetStatus
    @see getSetStatus
*/
typedef enum { SetOk, SetInvalidRange, SetInvalidPosition, SetNoMemory } SetStatus;

/** Crea un nuevo conjunto, sin elementos
    @param l El límite inferior del conjunto
    @param h El límite superior del conjunto
    @return Un nuevo conjunto
    @see Set
**/
Set createSet(int l, int h);

/** Crea un nuevo conjunto, con todos los elementos insertados
    Puede poner el estado de error a SetNoMemory, o SetInvalidRange.
    @param l El límite inferior del conjunto
    @param h El límite superior del conjunto
    @return Un nuevo conjunto
    @see Set, getSetStatus
**/
Set createFilledSet(int l, int h);


/**
    Devuelve si la el elemento pos está en el conjunto
    Puede poner el estado de error a SetInvalidPos
    @param s El conjunto que se va a comprobar
    @param pos La posición dentro del conjunto
    @return true si está fijada; false en otro caso
    @see getSetStatus
*/
bool querySetElement(Set s, int pos);

/**
    Inserta al elemento pos en el conjunto
    Puede poner el estado de error a SetInvalidPos
    @param s El conjunto que se va a cambiar
    @param pos El elemento dentro del conjunto
    @see getSetStatus
*/
void insertSetElement(Set s, int pos);

/**
    Elimina un elemento del conjunto
    Puede poner el estado de error a SetInvalidPos
    @param s El conjunto que se va a cambiar
    @param pos El elemento dentro del conjunto
    @see getSetStatus
*/
void deleteSetElement(Set s, int pos);

/**
    Devuelve el margen inferior de elementos dentro de un conjunto
    @param s El conjunto a comprobar
    @return el límite inferior, como un número entero
    @see getHighestPos
*/
inline
int getLowestElement(Set s)
    { return *( ( (int *) s ) + 1 ); }

/**
    Devuelve el margen superior de elementos dentro de un conjunto
    @param s El conjunto a comprobar
    @return el límite superior, como un número entero
    @see getLowestPos
*/
inline
int getHighestElement(Set s)
    { return *( (int *) s ); }

/**
    Comprueba si un elemento dado está dentro de los límites del conjunto
    @param s El conjunto a comprobar
    @return true si está dentro del margen inferior y superior; false en otro caso
    @see getLowestPos, getHighestPos
*/
bool isWithinBoundsOfSet(Set s, int pos);

/**
    Devuelve el estado del conjunto, dado por el último error sucedido
    @return El último error en la última operación.
    @see SetStatus
*/
SetStatus getSetStatus();

/**
    Vacía un determinado conjunto de elementos, en un rango.
    Puede poner el estado de error a SetInvalidPos
    @param s El conjunto a vaciar.
    @param l La posición inferior
    @param h La posición superior
    @see getSetStatus
*/
void deleteSetRange(Set s, int l, int h);

/**
    Vacía un determinado conjunto de elementos.
    @param s El conjunto a vaciar.
*/
inline
void deleteSet(Set s)
    { deleteSetRange( s, getLowestElement( s ), getHighestElement( s ) ); }

/**
    Llena el conjunto de elementos, en un rango dado.
    Puede poner el estado de error a SetInvalidPos
    @param s El conjunto a vaciar.
    @param l La posición inferior
    @param h La posición superior
    @see getSetStatus
*/
void insertSetRange(Set s, int l, int h);

/**
    Elimina un conjunto totalmente
*/
void destroySet(Set s);

#endif // SET_H_INCLUDED
